@@ -42,7 +42,8 @@ from .models.build_prophet import BuildProphet
 
 # Utils
 from .utils import colorful, load_ts_data, convert_timeseries_dataframe_to_supervised, \
-                   time_series_plot, print_static_rmse, print_dynamic_rmse, quick_ts_plot
+                   time_series_plot, print_static_rmse, print_dynamic_rmse, quick_ts_plot, \
+                   test_stationarity, print_ts_model_stats
 
 
 class auto_timeseries:
@@ -209,6 +210,9 @@ class auto_timeseries:
         start = time()
         print("Start of Fit.....")
 
+        ### first test the data for Stationary-ness #############
+        test_stationarity(traindata[target].values, plot=False, verbose=True)
+
         ##### Best hyper-parameters in statsmodels chosen using the best aic, bic or whatever. Select here.
         stats_scoring = 'aic'
 
@@ -288,6 +292,11 @@ class auto_timeseries:
 
         preds = [x for x in list(ts_df) if x not in [self.ts_column, target]]
 
+        try:
+            time_series_plot(ts_df[target], lags=self.seasonal_period, title='Original Time Series',
+                    chart_type='line', chart_freq=self.time_interval)
+        except:
+            print('Could not draw time series plot of data set. Continuing')
         ##################################################################################################
         ### Turn the time series index into a variable and calculate the difference.
         ### If the difference is not in days, then it is a hourly or minute based time series
@@ -565,54 +574,57 @@ class auto_timeseries:
         if self.__any_contained_in_list(what_list=['var','Var','VAR', 'stats', 'best'], in_list=self.model_type):
             ########### Let's build a VAR Model - but first we have to shift the predictor vars ####
 
-            print("\n")
-            print("="*50)
-            print("Building VAR Model")
-            print("="*50)
-            print("\n")
-
-            name = 'VAR'
-            # Placeholder for cases when model can not be built
-            score_val = np.inf
-            model_build = None
-            model = None
-            forecasts = None
-
-            if len(preds) == 0:
-                print(colorful.BOLD + '\nNo VAR model created since no explanatory variables given in data set' + colorful.END)
+            if ts_df.shape[0] > 1000 and self.__any_contained_in_list(what_list=['stats', 'best'], in_list=self.model_type):
+                print("Skipping VAR Model since dataset is > 1000 rows and it will take too long")
             else:
-                try:
-                    print(colorful.BOLD + '\nRunning VAR Model...' + colorful.END)
-                    print('    Shifting %d predictors by 1 to align prior predictor values with current target values...'
-                                            %len(preds))
+                print("\n")
+                print("="*50)
+                print("Building VAR Model - best suited for small datasets < 1000 rows and < 10 columns")
+                print("="*50)
+                print("\n")
 
-                    # TODO: This causes an issue later in ML (most likely cause of https://github.com/AutoViML/Auto_TS/issues/15)
-                    # Since we are passing ts_df there. Make sure you don't assign it
-                    # back to the same variable. Make a copy and make changes to that copy.
-                    ts_df_shifted = ts_df.copy(deep=True)
-                    ts_df_shifted[preds] = ts_df_shifted[preds].shift(1)
-                    ts_df_shifted.dropna(axis=0,inplace=True)
+                name = 'VAR'
+                # Placeholder for cases when model can not be built
+                score_val = np.inf
+                model_build = None
+                model = None
+                forecasts = None
 
-                    model_build = BuildVAR(scoring=stats_scoring, forecast_period=self.forecast_period, p_max=p_max, q_max=q_max)
-                    model, forecasts, rmse, norm_rmse = model_build.fit(
-                        ts_df_shifted[[target]+preds],
-                        target_col=target,
-                        cv = cv
-                    )
+                if len(preds) == 0:
+                    print(colorful.BOLD + '\nNo VAR model created since no explanatory variables given in data set' + colorful.END)
+                else:
+                    try:
+                        print(colorful.BOLD + '\nRunning VAR Model...' + colorful.END)
+                        print('    Shifting %d predictors by 1 to align prior predictor values with current target values...'
+                                                %len(preds))
 
-                    if self.score_type == 'rmse':
-                        score_val = rmse
-                    else:
-                        score_val = norm_rmse
-                except Exception as e:
-                    print("Exception occurred while building VAR model...")
-                    print(e)
-                    warnings.warn('    VAR model error: predictions not available.')
+                        # TODO: This causes an issue later in ML (most likely cause of https://github.com/AutoViML/Auto_TS/issues/15)
+                        # Since we are passing ts_df there. Make sure you don't assign it
+                        # back to the same variable. Make a copy and make changes to that copy.
+                        ts_df_shifted = ts_df.copy(deep=True)
+                        ts_df_shifted[preds] = ts_df_shifted[preds].shift(1)
+                        ts_df_shifted.dropna(axis=0,inplace=True)
 
-            self.ml_dict[name]['model'] = model
-            self.ml_dict[name]['forecast'] = forecasts
-            self.ml_dict[name][self.score_type] = score_val
-            self.ml_dict[name]['model_build'] = model_build
+                        model_build = BuildVAR(scoring=stats_scoring, forecast_period=self.forecast_period, p_max=p_max, q_max=q_max)
+                        model, forecasts, rmse, norm_rmse = model_build.fit(
+                            ts_df_shifted[[target]+preds],
+                            target_col=target,
+                            cv = cv
+                        )
+
+                        if self.score_type == 'rmse':
+                            score_val = rmse
+                        else:
+                            score_val = norm_rmse
+                    except Exception as e:
+                        print("Exception occurred while building VAR model...")
+                        print(e)
+                        warnings.warn('    VAR model error: predictions not available.')
+
+                self.ml_dict[name]['model'] = model
+                self.ml_dict[name]['forecast'] = forecasts
+                self.ml_dict[name][self.score_type] = score_val
+                self.ml_dict[name]['model_build'] = model_build
 
         if self.__any_contained_in_list(what_list=['ml', 'ML','best'], in_list=self.model_type):
             ########## Let's build a Machine Learning Model now with Time Series Data ################
@@ -915,7 +927,7 @@ class auto_timeseries:
         return all([True if elem in in_list else False for elem in what_list])
 #################################################################################
 module_type = 'Running' if  __name__ == "__main__" else 'Imported'
-version_number = '0.0.27'
+version_number = '0.0.28'
 print(f"""{module_type} auto_timeseries version:{version_number}. Call by using:
 model = auto_timeseries(score_type='rmse', forecast_period=forecast_period,
                 time_interval='Month',
@@ -924,6 +936,6 @@ model = auto_timeseries(score_type='rmse', forecast_period=forecast_period,
                 verbose=2)
 
 model.fit(traindata, ts_column,target)
-model.predict(testdata, forecast_period)
+model.predict(testdata, model='best')
 """)
 #################################################################################
