@@ -2,7 +2,7 @@ from typing import Tuple
 import numpy as np  # type: ignore
 import pandas as pd  # type: ignore
 import matplotlib.pyplot as plt # type: ignore
-from sklearn.metrics import mean_absolute_error, mean_squared_error # type: ignore  
+from sklearn.metrics import mean_absolute_error, mean_squared_error # type: ignore
 
 
 def print_static_rmse(actual: np.array, predicted: np.array, start_from: int=0, verbose: int=0) -> Tuple[float, float]:
@@ -13,7 +13,7 @@ def print_static_rmse(actual: np.array, predicted: np.array, start_from: int=0,
     """
     rmse = np.sqrt(mean_squared_error(actual[start_from:], predicted[start_from:]))
     std_dev = actual[start_from:].std()
-    if verbose == 1:
+    if verbose >= 1:
         print('    RMSE = %0.2f' %rmse)
         print('    Std Deviation of Actuals = %0.2f' %(std_dev))
         print('    Normalized RMSE = %0.1f%%' %(rmse*100/std_dev))
@@ -63,35 +63,37 @@ def print_mape(y: np.array, y_hat: np.array) -> float:
     """
     Calculating Mean Absolute Percent Error https://en.wikipedia.org/wiki/Mean_absolute_percentage_error
     """
-    perc_err = (100*(y - y_hat))/y
-    return np.mean(abs(perc_err))
+    try:
+        perc_err = (100*(y - y_hat))/y
+        return np.mean(abs(perc_err))
+    except:
+        return np.nan
 
-
-def print_ts_model_stats(actuals: np.array, predicted: np.array, number_as_percentage:float =100) -> Tuple[float, float, float]:
+def print_ts_model_stats(actuals: np.array, predicted: np.array) -> Tuple[float, float, float]:
     """
     This program prints and returns MAE, RMSE, MAPE.
     If you like the MAE and RMSE as a percentage of something, just give that number
     in the input as "number_as_percentage" and it will return the MAE and RMSE as a
     ratio of that number. Returns MAE, MAE_as_percentage, and RMSE_as_percentage
     """
-    #print(len(actuals))
-    #print(len(predicted))
+    number_as_percentage = actuals.std()
     plt.figure(figsize=(15,8))
-    dfplot = pd.DataFrame([predicted,actuals]).T
-    dfplot.columns = ['Forecast','Actual']
+    dfplot = pd.DataFrame([actuals,predicted]).T
+    dfplot.columns = ['Actual','Forecast']
     plt.plot(dfplot)
-    plt.legend(['Forecast','Actual'])
+    plt.legend(['actual','forecast'])
+    plt.title('Random Forest: Actual vs Forecast in expanding (training) Window Cross Validation', fontsize=20)
+    print('\n-------------------------------------------')
+    print('Model Cross Validation Results:')
+    print('-------------------------------------------')
     mae = mean_absolute_error(actuals, predicted)
     mae_asp = (mean_absolute_error(actuals, predicted)/number_as_percentage)*100
-    rmse_asp = (np.sqrt(mean_squared_error(actuals,predicted))/number_as_percentage)*100
-    print('MAE (%% AUM) = %0.2f%%' %mae_asp)
-    print('RMSE (%% AUM) = %0.2f%%' %rmse_asp)
-    print('MAE (as %% Actual) = %0.2f%%' %(mae/abs(actuals).mean()*100))
-    _ = print_mape(actuals, predicted)
+    print('    MAE (as %% Std Dev of Actuals) = %0.2f%%' %mae_asp)
     rmse = print_rmse(actuals, predicted)
     mape = print_mape(actuals, predicted)
-    print("MAPE = %0.0f%%" %(mape))
+    print("    MAPE (Mean Absolute Percent Error) = %0.0f%%" %(mape))
     # Normalized RMSE print('RMSE = {:,.Of}'.format(rmse))
-    print('Normalized RMSE (MinMax) = %0.0f%%' %(100*rmse/abs(actuals.max()-actuals.min())))
-    print('Normalized RMSE = %0.0f%%' %(100*rmse/actuals.std()))
-    return mae, mae_asp, rmse_asp
+    print('    Normalized RMSE (MinMax) = %0.0f%%' %(100*rmse/abs(actuals.max()-actuals.min())))
+    rmse_asp = (np.sqrt(mean_squared_error(actuals,predicted))/number_as_percentage)*100
+    print('    Normalized RMSE (as Std Dev of Actuals)= %0.0f%%' %rmse_asp)
+    return rmse, rmse_asp
